/**
* The Society of Actuaries (SOA) makes this computer model and its computer
* code available to its members for educational purposes. The model and its
* code may be modified and redistributed without prior authorization from the
* SOA. The SOA distributes this computer model and its computer code in the
* hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
* implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
* PURPOSE.
*/ 

package agents;

import resources.AgentID;
import resources.Constants.GoalDescription;

//==============================================================================
//Import statements
//------------------------------------------------------------------------------

public abstract class Agent
{
//==============================================================================
// Attributes
// These attribute variables are available to other classes
// through "getter" methods.
//------------------------------------------------------------------------------
    private AgentID agentID;
    private double agentXLocation;
    private double agentYLocation;
    private GoalDescription agentFirstPriorityGoalDescription;
    private GoalDescription agentSecondPriorityGoalDescription; 
//==============================================================================

//==============================================================================
// Memory
// These memory variables are available to other classes
// through "getter" methods. 
//------------------------------------------------------------------------------    

// None

//==============================================================================

//==============================================================================
// Other variables
// These are other variables used in the class. 
//------------------------------------------------------------------------------

// None

//==============================================================================


//==============================================================================
// Methods: Initialization methods for the agent. These are used in
//          the agent's constructor method or at the beginning of each
//          simulation period.
//==============================================================================

// None

//==============================================================================


//==============================================================================
// Methods:  "Input" methods for the agent. These are described in the
// detailed description of the model. Each method is preceded by "i#".
//==============================================================================

// None

//==============================================================================


//==============================================================================
// Methods:  "Process" methods for the agent. These are described in the
// detailed description of the model. Each method is preceded
// by "p#" to indicate its number according to the numbering scheme used in the
// detailed description.
//==============================================================================

// None

//==============================================================================


//==============================================================================
// Methods:  "Rule" methods for the agent. These are described in the
// detailed description of the model. Each method is preceded
// by "r#" to indicate its number according to the numbering scheme used in the
// detailed description.
//==============================================================================

// None

//==============================================================================


//==============================================================================
// Methods:  General methods for the agent.
//==============================================================================

//==============================================================================
// Method:  distance()
// Calculates the distance between two points.
// It is used in subclasses, and so has "protected" scope.
//------------------------------------------------------------------------------
    protected double distance(double pLocation1X, double pLocation1Y,
            double pLocation2X, double pLocation2Y)
    {
        double distance;
        
        distance = Math.pow(Math.pow(pLocation1X - pLocation2X, 2)
                + Math.pow(pLocation1Y - pLocation2Y, 2), 0.5);
        
        return distance;
        
    }//end distance()
//==============================================================================


//==============================================================================
// Methods: Methods for the model user to probe the agent's data.
// Note:    As a simulation runs, the model user can probe an agent by
//          double-clicking on its icon in the display.
//==============================================================================

// None

//==============================================================================


//==============================================================================
// Methods:  "Chart" methods to support simulation charts and graphs.
//==============================================================================

// None

//==============================================================================


//==============================================================================
// Methods:  General methods for the agent.
//==============================================================================

// None

//==============================================================================


//==============================================================================
// Methods:  getter and setter methods
//------------------------------------------------------------------------------
    /**
     * Returns the Agent's identifier.
     *
     * @return AgentID agentID.
     */
    public AgentID getAgentID()
    {
        return (agentID);
    }

    /**
     * Sets the Agent's identifier.
     *
     */
    public void setAgentID(AgentID pAgentID)
    {
        agentID = pAgentID;
    }

    /**
     * Returns the Agent's X location.
     *
     * @return double agentXLocation.
     */
    public double getAgentXLocation()
    {
        return (agentXLocation);
    }

    /**
     * Returns the Agent's Y location.
     *
     * @return double agentYLocation.
     */
    public double getAgentYLocation()
    {
        return (agentYLocation);
    }

    /**
     * Sets the Agent's X location.
     *
     */
    public void setAgentXLocation(double pXLocation)
    {
        agentXLocation = pXLocation;
        
    }
    
    /**
     * Sets the Agent's Y location.
     *
     */
    public void setAgentYLocation(double pYLocation)
    {
        agentYLocation = pYLocation;
        
    }
    
    /**
     * @return the agentFirstPriorityGoalDescription
     */
    public GoalDescription getAgentFirstPriorityGoalDescription()
    {
        return agentFirstPriorityGoalDescription;
    }

    /**
     * @param agentFirstPriorityGoalDescription the agentFirstPriorityGoalDescription to set
     */
    public void setAgentFirstPriorityGoalDescription(GoalDescription agentFirstPriorityGoalDescription)
    {
        this.agentFirstPriorityGoalDescription = agentFirstPriorityGoalDescription;
    }   

    /**
     * @return the agentSecondPriorityGoalDescription
     */
    public GoalDescription getAgentSecondPriorityGoalDescription()
    {
        return agentSecondPriorityGoalDescription;
    }

    /**
     * @param agentSecondPriorityGoalDescription the agentSecondPriorityGoalDescription to set
     */
    public void setAgentSecondPriorityGoalDescription(GoalDescription agentSecondPriorityGoalDescription)
    {
        this.agentSecondPriorityGoalDescription = agentSecondPriorityGoalDescription;
    }
//==============================================================================

    
}//end Agent class

