/**
* The Society of Actuaries (SOA) makes this computer model and its computer
* code available to its members for educational purposes. The model and its
* code may be modified and redistributed without prior authorization from the
* SOA. The SOA distributes this computer model and its computer code in the
* hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
* implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
* PURPOSE.
*/ 

package agents;

//==============================================================================
//Import statements
//------------------------------------------------------------------------------
import implementation.Schedule;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.ListIterator;
import jxl.*;
import jxl.read.biff.BiffException;
import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.parameter.Parameters;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.continuous.ContinuousSpace;
import resources.AgentID;
import resources.Constants;
import resources.Constants.AgentType;
import resources.Constants.IntensityLevel;


/**
 * @author      Alan Mills
 * <p>
 * This class implements the simulation's Environment. The Environment is the
 * container for the models agents. It creates the simulations Employee agents,
 * and maintains a list of them. It also reads data from external files, 
 * obtains and validates user-provided parameters, and schedules agent behaviors.
 * @version     1.0
 *
 */
@SuppressWarnings("unused")
public class Environment
{
//==============================================================================
// Attribute variables
// These variables describe the agent at a current point in time.
// They are available to other classes through "getter" methods. 
//------------------------------------------------------------------------------

// None

//==============================================================================


//==============================================================================
// Parameter variables
// These variables are available to other classes through "getter" methods.
// They are the parameters entered by the user through the Repast GUI.
// The identifier after the variable name refers to the identifier on the
// GUI parameter screen.
//------------------------------------------------------------------------------
    private static String employeeLabels; //A1
    private static int maximumSimulationPeriods; //A2
    private static int randomSeed; //B
    private static String programDesign_TargetGroup; //C1
    private static IntensityLevel programDesign_Type; //C2a
    private static double programDesign_TypeLevel1AnnualCost; //C2b
    private static double programDesign_TypeLevel2AnnualCost; //C2c
    private static IntensityLevel programDesign_ChoiceArchitectureIntensity; //C3a
    private static double programDesign_ChoiceArchitectureLevel1AnnualCost; //C3b
    private static double programDesign_ChoiceArchitectureLevel2AnnualCost; //C3c
    private static IntensityLevel programDesign_IncentiveIntensity; //C4a
    private static double programDesign_IncentiveLevel1AnnualCost; //C4b
    private static double programDesign_IncentiveLevel2AnnualCost; //C4c
    private static double averageAnnualDiabetesMedicalExpenditure; //D
    private static int numberOfEmployees; //E1
    private static String employeeAgeDistributionType; //E2
    private static int employeeAverageAge; //E3
    private static double employeeAgeStandardDeviation; //E4
    private static String employeeWorkplaceLocationDistributionType; //E5
    private static double employeeWorkplaceLocationStandardDeviation; //E6
    private static double employeeWorkplaceNeighborhoodRadius; //F
    private static double employeeFirstGoalPercent_Health; //G1
    private static double employeeFirstGoalPercent_Conformance; //G2
    private static double employeeFirstGoalPercent_Income; //G3   
    private static double employeesNormalWeightPercent; //H1
    private static double employeesOverweightPercent; //H2
    private static double employeesObesePercent; //H3
    private static double employeeStageOfChange_NormalWeight_Stage1Percent; //I1a
    private static double employeeStageOfChange_NormalWeight_Stage2Percent; //I1b 
    private static double employeeStageOfChange_NormalWeight_Stage3Percent; //I1c
    private static double employeeStageOfChange_Overweight_Stage1Percent; //I2a
    private static double employeeStageOfChange_Overweight_Stage2Percent; //I2b
    private static double employeeStageOfChange_Overweight_Stage3Percent; //I2c    
    private static double employeeStageOfChange_Obese_Stage1Percent; //I3a
    private static double employeeStageOfChange_Obese_Stage2Percent; //I3b
    private static double employeeStageOfChange_Obese_Stage3Percent; //I3c
    private static double turnover_BaseProbability; //J1
    private static double turnover_AdditionalProbability; //J2
    private static int retirementAge; //J3
    private static double absenteeismDays; //K1
    private static double presenteeismDays; //K2
    private static String dataWorkbookFileName; //L1
    private static String diabetesIncidenceWorksheetName; //L2a
    private static double diabetesIncidenceAdjustmentFactor; //L2b
    private static String diabetesPrevalenceWorksheetName; //L3a
    private static double diabetesPrevalenceAdjustmentFactor; //L3b
    private static String diabetesRemissionWorksheetName; //L4a
    private static double diabetesRemissionAdjustmentFactor; //L4b
    private static String weightProgressionWorksheetName; //L5a
    private static double weightProgressionAdjustmentFactor; //L5b
    private static String weightRegressionWorksheetName; //L6a
    private static double weightRegressionAdjustmentFactor; //L6b
    private static String overrideParameters; //M1
    private static String writeOutputFile; //M2
    private static String outputFileName; //M3

//==============================================================================
// Data variables
// Variables for data read in from Excel data files.
// They are available to other classes through "getter" methods.
//------------------------------------------------------------------------------
    private static double[][] diabetesIncidenceRate = 
            new double[Constants.DATA_TABLE_MAXIMUM_AGE][Constants.NUMBER_OF_WEIGHT_CATEGORIES];
    private static double[][] diabetesPrevalenceRate = 
            new double[Constants.DATA_TABLE_MAXIMUM_AGE][Constants.NUMBER_OF_WEIGHT_CATEGORIES];
    private static double[][] diabetesRemissionRate = 
            new double[Constants.DATA_TABLE_MAXIMUM_AGE][Constants.NUMBER_OF_WEIGHT_CATEGORIES];
    private static double[][] weightProgressionRate = 
            new double[Constants.DATA_TABLE_MAXIMUM_AGE][Constants.NUMBER_OF_WEIGHT_CATEGORIES];
    private static double[][] weightRegressionRate = 
            new double[Constants.DATA_TABLE_MAXIMUM_AGE][Constants.NUMBER_OF_WEIGHT_CATEGORIES];
                      
//=============================================================================


    
//==============================================================================
// Memory variables
// These variables include changing information over time.
// They are available to other classes through "getter" methods. 
//------------------------------------------------------------------------------
    private static ArrayList<Employee> employeeList = new ArrayList<Employee>();
    private static int runNumber = 0;   
//==============================================================================

//==============================================================================
// Other variables
// These are other variables used in the class.
// They are not available through "getter" methods.
//------------------------------------------------------------------------------
    static double xCoordinate;
    static double yCoordinate;
    private static ContinuousSpace<Object> workplaceSpace;
    private Context<Object> simulationContext;
    static int currentMaximumEmployeeIDNumber;

 
//==============================================================================

//==============================================================================
// Method:  Environment() constructor
//------------------------------------------------------------------------------
     /**
      * Constructs the Environment agent.
      *
      * The constructor obtains and validates input parameters,
      * reads data from the Excel file, creates the Employee agents,
      * creates a list of the Employee agents, and schedules Employee behaviors.
      * 
      * @param pWorkplaceSpace The identifier for the "space" simulation environment
      * @param pContext The identifier for the simulation "context"
     * @throws BiffException 
      */
     public Environment(ContinuousSpace<Object> pWorkplaceSpace, Context<Object> pContext)
     {
         simulationContext = pContext;
         workplaceSpace = pWorkplaceSpace;
  
         // Increment and print out the static run number.
         // Note that this static variable is not initialized.
         runNumber++;
         System.out.println("Run number: " + runNumber); //Useful for batch runs
         System.out.println("======================================");

         //======================================================================
         // Get the input parameters from the Repast GUI
         // and test them for validity.
         //----------------------------------------------------------------------
         i1_getInputParameters();

         //======================================================================
         // Get data from the Excel file
         // and test then for validity.
         //----------------------------------------------------------------------
         i2_getExcelData();
         
        //----------------------------------------------------------------------
        // Override parameters.
        // This section of the constructor is used to override user-entered
        // parameters. It is turned on by the parameter "M1. Override parameters".
        // Note: Sometimes the "parameter sweep" feature
        // cannot properly set up the required parameters.
        // Nevertheless, even in these cases, the "parameter sweep" feature
        // can be used to execute the required number of batch runs.
        // Alternatively, the program can write out a custom file (see the
        // "WriteOutput()" method in the Schedule class).
        //---------------------------------------------------------------------- 
        if (overrideParameters.matches("Yes"))
        {
            p1_OverrideParameters();

        }//end if         

        //----------------------------------------------------------------------
        // Create the Employee agents, add them to the simulation's "Context",
        // and add them to the "Employee list".
        //----------------------------------------------------------------------

        if (employeeList != null) employeeList.clear();
        
        for (int i = 0; i < numberOfEmployees; i++)
        {
            p2_CreateEmployee(i, simulationContext);

        }//end for
        

        //----------------------------------------------------------------------
        // Schedule agent behaviors (methods)
        //-----------------------------
        Schedule agentSchedule = new Schedule(simulationContext);
    
     }//end Environment()
//==============================================================================


//==============================================================================
// Method:  i1_getInputParameters()
//------------------------------------------------------------------------------

     public void i1_getInputParameters()
     {

         Parameters parameters = RunEnvironment.getInstance().getParameters();

         employeeLabels =
                 (String) parameters.getValue("employeeLabels"); //A1
         maximumSimulationPeriods =
                 (Integer) parameters.getValue("maximumSimulationPeriods"); //A2
         randomSeed =
                 (Integer) parameters.getValue("randomSeed"); //B     
         programDesign_TargetGroup =
                 (String)parameters.getValue("programDesign_TargetGroup"); //C1
         
         if (((String) parameters.getValue("programDesign_Type")).equals("None"))
             programDesign_Type = IntensityLevel.None; //C2a
         if (((String) parameters.getValue("programDesign_Type")).equals("Level 1"))
             programDesign_Type = IntensityLevel.Level1; //C2a
         if (((String) parameters.getValue("programDesign_Type")).equals("Level 2"))
             programDesign_Type = IntensityLevel.Level2; //C2a
         
         programDesign_TypeLevel1AnnualCost =
                 (Double) parameters.getValue("programDesign_TypeLevel1AnnualCost"); //C2b
         programDesign_TypeLevel2AnnualCost =
                 (Double) parameters.getValue("programDesign_TypeLevel2AnnualCost"); //C2c
         
         if (((String) parameters.getValue("programDesign_ChoiceArchitectureIntensity")).equals("None"))
             programDesign_ChoiceArchitectureIntensity = IntensityLevel.None; //C3a
         if (((String) parameters.getValue("programDesign_ChoiceArchitectureIntensity")).equals("Level 1"))
             programDesign_ChoiceArchitectureIntensity = IntensityLevel.Level1; //C3a
         if (((String) parameters.getValue("programDesign_ChoiceArchitectureIntensity")).equals("Level 2"))
             programDesign_ChoiceArchitectureIntensity = IntensityLevel.Level2; //C3a
         
         programDesign_ChoiceArchitectureLevel1AnnualCost =
                 (Double) parameters.getValue("programDesign_ChoiceArchitectureLevel1AnnualCost"); //C3b
         programDesign_ChoiceArchitectureLevel2AnnualCost =
                 (Double) parameters.getValue("programDesign_ChoiceArchitectureLevel2AnnualCost"); //C3c
         
         if (((String) parameters.getValue("programDesign_IncentiveIntensity")).equals("None"))
             programDesign_IncentiveIntensity = IntensityLevel.None; //C4a
         if (((String) parameters.getValue("programDesign_IncentiveIntensity")).equals("Level 1"))
             programDesign_IncentiveIntensity = IntensityLevel.Level1; //C4a
         if (((String) parameters.getValue("programDesign_IncentiveIntensity")).equals("Level 2"))
             programDesign_IncentiveIntensity = IntensityLevel.Level2; //C4a
         
         programDesign_IncentiveLevel1AnnualCost =
                 (Double) parameters.getValue("programDesign_IncentiveLevel1AnnualCost"); //C4b
         programDesign_IncentiveLevel2AnnualCost =
                 (Double) parameters.getValue("programDesign_IncentiveLevel2AnnualCost"); //C4c
         averageAnnualDiabetesMedicalExpenditure =
                 (Double) parameters.getValue("averageAnnualDiabetesMedicalExpenditure"); //D
         numberOfEmployees =
                 (Integer) parameters.getValue("numberOfEmployees"); //E1
         employeeAgeDistributionType =
                 (String) parameters.getValue("employeeAgeDistributionType"); //E2
         employeeAverageAge =
                 (Integer) parameters.getValue("employeeAverageAge"); //E3
         employeeAgeStandardDeviation =
                 (Double) parameters.getValue("employeeAgeStandardDeviation"); //E4
         employeeWorkplaceLocationDistributionType =
                 (String) parameters.getValue("employeeWorkplaceLocationDistributionType"); //E5
         employeeWorkplaceLocationStandardDeviation =
                 (Double) parameters.getValue("employeeWorkplaceLocationStandardDeviation"); //E6
         employeeWorkplaceNeighborhoodRadius =
                 (Double) parameters.getValue("employeeWorkplaceNeighborhoodRadius"); //F
         employeeFirstGoalPercent_Health =
                 (Double) parameters.getValue("employeeFirstGoalPercent_Health"); //G1
         employeeFirstGoalPercent_Conformance =
                 (Double) parameters.getValue("employeeFirstGoalPercent_Conformance"); //G2
         employeeFirstGoalPercent_Income =
                 (Double) parameters.getValue("employeeFirstGoalPercent_Income"); //G3
         employeesNormalWeightPercent =
                 (Double) parameters.getValue("employeesNormalWeightPercent"); //H1
         employeesOverweightPercent =
                 (Double) parameters.getValue("employeesOverweightPercent"); //H2
         employeesObesePercent =
                 (Double) parameters.getValue("employeesObesePercent"); //H3
         employeeStageOfChange_NormalWeight_Stage1Percent =
                 (Double) parameters.getValue("employeeStageOfChange_NormalWeight_Stage1Percent"); //I1a
         employeeStageOfChange_NormalWeight_Stage2Percent =
                 (Double) parameters.getValue("employeeStageOfChange_NormalWeight_Stage2Percent"); //I1b 
         employeeStageOfChange_NormalWeight_Stage3Percent =
                 (Double) parameters.getValue("employeeStageOfChange_NormalWeight_Stage3Percent"); //I1c
         employeeStageOfChange_Overweight_Stage1Percent =
                 (Double) parameters.getValue("employeeStageOfChange_Overweight_Stage1Percent"); //I2a
         employeeStageOfChange_Overweight_Stage2Percent =
                 (Double) parameters.getValue("employeeStageOfChange_Overweight_Stage2Percent"); //I2b
         employeeStageOfChange_Overweight_Stage3Percent =
                 (Double) parameters.getValue("employeeStageOfChange_Overweight_Stage3Percent"); //I2c
         employeeStageOfChange_Obese_Stage1Percent =
                 (Double) parameters.getValue("employeeStageOfChange_Obese_Stage1Percent"); //I3a
         employeeStageOfChange_Obese_Stage2Percent =
                 (Double) parameters.getValue("employeeStageOfChange_Obese_Stage2Percent"); //I3b
         employeeStageOfChange_Obese_Stage3Percent =
                 (Double) parameters.getValue("employeeStageOfChange_Obese_Stage3Percent"); //I3c
         turnover_BaseProbability =
                 (Double) parameters.getValue("turnover_BaseProbability"); //J1
         turnover_AdditionalProbability =
                 (Double) parameters.getValue("turnover_AdditionalProbability"); //J2
         retirementAge =
                 (Integer) parameters.getValue("retirementAge"); //J3
         absenteeismDays =
                 (Double) parameters.getValue("absenteeismDays"); //K1
         presenteeismDays =
                 (Double) parameters.getValue("presenteeismDays"); //K2
         dataWorkbookFileName =
                 (String) parameters.getValue("dataWorkbookFileName"); //L1
         diabetesIncidenceWorksheetName =
                 (String) parameters.getValue("diabetesIncidenceWorksheetName"); //L2a
         diabetesIncidenceAdjustmentFactor =
                 (Double) parameters.getValue("diabetesIncidenceAdjustmentFactor"); //L2b
         diabetesPrevalenceWorksheetName =
                 (String) parameters.getValue("diabetesPrevalenceWorksheetName"); //L3a
         diabetesPrevalenceAdjustmentFactor =
                 (Double) parameters.getValue("diabetesPrevalenceAdjustmentFactor"); //L3b
         diabetesRemissionWorksheetName =
                 (String) parameters.getValue("diabetesRemissionWorksheetName"); //L4a
         diabetesRemissionAdjustmentFactor =
                 (Double) parameters.getValue("diabetesRemissionAdjustmentFactor"); //L4b
         weightProgressionWorksheetName =
                 (String) parameters.getValue("weightProgressionWorksheetName"); //L5a
         weightProgressionAdjustmentFactor =
                 (Double) parameters.getValue("weightProgressionAdjustmentFactor"); //L5b
         weightRegressionWorksheetName =
                 (String) parameters.getValue("weightRegressionWorksheetName"); //L6a
         weightRegressionAdjustmentFactor =
                 (Double) parameters.getValue("weightRegressionAdjustmentFactor"); //L6b
         overrideParameters =
                 (String) parameters.getValue("overrideParameters"); //M1
         writeOutputFile =
                 (String) parameters.getValue("writeOutputFile"); //M2
         outputFileName =
                 (String) parameters.getValue("outputFileName"); //M3  

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Test 
// This block tests the user-entered parameter values.
// To run the test, remove comments from the code below.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  System.out.println("Environment i1_getInputParameters() at " +
//       RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
//
//  System.out.println("employeeLabels:  " +
//          getEmployeeLabels()); //A1
//  System.out.println("maximumSimulationPeriods:  " +
//          getMaximumSimulationPeriods()); //A2
//  System.out.println("randomSeed:  " +
//          getRandomSeed()); //B
//  System.out.println("programDesign_TargetGroup:  " +
//          getProgramDesign_TargetGroup()); //C1
//  System.out.println("programDesign_Type:  " +
//          getProgramDesign_Type()); //C2a
//  System.out.println("programDesign_TypeLevel1AnnualCost:  " +
//          getProgramDesign_TypeLevel1AnnualCost()); //C2b
//  System.out.println("programDesign_TypeLevel2AnnualCost:  " +
//          getProgramDesign_TypeLevel2AnnualCost()); //C2c
//  System.out.println("programDesign_ChoiceArchitectureIntensity:  " +
//          getProgramDesign_ChoiceArchitectureIntensity()); //C3a
//  System.out.println("programDesign_ChoiceArchitectureLevel1AnnualCost:  " +
//          getProgramDesign_ChoiceArchitectureLevel1AnnualCost()); //C3b
//  System.out.println("programDesign_ChoiceArchitectureLevel2AnnualCost:  " +
//          getProgramDesign_ChoiceArchitectureLevel2AnnualCost()); //C3c
//  System.out.println("programDesign_IncentiveIntensity:  " +
//          getProgramDesign_IncentiveIntensity()); //C4a
//  System.out.println("programDesign_IncentiveLevel1AnnualCost:  " +
//          getProgramDesign_IncentiveLevel1AnnualCost()); //C4b
//  System.out.println("programDesign_IncentiveLevel2AnnualCost:  " +
//          getProgramDesign_IncentiveLevel2AnnualCost()); //C4c
//  System.out.println("averageAnnualDiabetesMedicalExpenditure:  " +
//          getAverageAnnualDiabetesMedicalExpenditure()); //D
//  System.out.println("numberOfEmployees:  " +
//          getNumberOfEmployees()); //E1
//  System.out.println("employeeAgeDistributionType:  " +
//          getEmployeeAgeDistributionType()); //E2
//  System.out.println("employeeAverageAge:  " +
//          getEmployeeAverageAge()); //E3
//  System.out.println("employeeAgeStandardDeviation:  " +
//          getEmployeeAgeStandardDeviation()); //E4
//  System.out.println("employeeWorkplaceLocationDistributionType:  " +
//          getEmployeeWorkplaceLocationDistributionType()); //E5
//  System.out.println("employeeWorkplaceLocationStandardDeviation:  " +
//          getEmployeeWorkplaceLocationStandardDeviation()); //E6
//  System.out.println("employeeWorkplaceNeighborhoodRadius:  " +
//          getEmployeeWorkplaceNeighborhoodRadius()); //F
//  System.out.println("employeeFirstGoalPercent_Health:  " +
//          getEmployeeFirstGoalPercent_Health()); //G1
//  System.out.println("employeeFirstGoalPercent_Conformance:  " +
//          getEmployeeFirstGoalPercent_Conformance()); //G2
//  System.out.println("employeeFirstGoalPercent_Income:  " +
//          getEmployeeFirstGoalPercent_Income()); //G3   
//  System.out.println("employeesNormalWeightPercent:  " +
//          getEmployeesNormalWeightPercent()); //H1
//  System.out.println("employeesOverweightPercent:  " +
//          getEmployeesOverweightPercent()); //H2
//  System.out.println("employeesObesePercent:  " +
//          getEmployeesObesePercent()); //H3
//  System.out.println("employeeStageOfChange_NormalWeight_Stage1Percent:  " +
//          getEmployeeStageOfChange_NormalWeight_Stage1Percent()); //I1a
//  System.out.println("employeeStageOfChange_NormalWeight_Stage2Percent:  " +
//          getEmployeeStageOfChange_NormalWeight_Stage2Percent()); //I1b 
//  System.out.println("employeeStageOfChange_NormalWeight_Stage3Percent:  " +
//          getEmployeeStageOfChange_NormalWeight_Stage3Percent()); //I1c
//  System.out.println("employeeStageOfChange_Overweight_Stage1Percent:  " +
//          getEmployeeStageOfChange_Overweight_Stage1Percent()); //I2a
//  System.out.println("employeeStageOfChange_Overweight_Stage2Percent:  " +
//          getEmployeeStageOfChange_Overweight_Stage2Percent()); //I2b
//  System.out.println("employeeStageOfChange_Overweight_Stage3Percent:  " +
//          getEmployeeStageOfChange_Overweight_Stage3Percent()); //I2c    
//  System.out.println("employeeStageOfChange_Obese_Stage1Percent:  " +
//          getEmployeeStageOfChange_Obese_Stage1Percent()); //I3a
//  System.out.println("employeeStageOfChange_Obese_Stage2Percent:  " +
//          getEmployeeStageOfChange_Obese_Stage2Percent()); //I3b
//  System.out.println("employeeStageOfChange_Obese_Stage3Percent:  " +
//          getEmployeeStageOfChange_Obese_Stage3Percent()); //I3c       
//  System.out.println("turnover_BaseProbability:  " +
//          getTurnover_BaseProbability()); //J1
//  System.out.println("turnover_AdditionalProbability:  " +
//          getTurnover_AdditionalProbability()); //J2
//  System.out.println("retirementAge:  " +
//          getRetirementAge()); //J3
//  System.out.println("absenteeismDays:  " +
//          getAbsenteeismDays()); //K1
//  System.out.println("presenteeismDays:  " +
//          getPresenteeismDays()); //K2
//  System.out.println("dataWorkbookFileName:  " +
//          getDataWorkbookFileName()); //L1
//  System.out.println("diabetesIncidenceWorksheetName:  " +
//          getDiabetesIncidenceWorksheetName()); //L2a
//  System.out.println("diabetesIncidenceAdjustmentFactor:  " +
//          getDiabetesIncidenceAdjustmentFactor()); //L2b
//  System.out.println("diabetesPrevalenceWorksheetName:  " +
//          getDiabetesPrevalenceWorksheetName()); //L3a
//  System.out.println("diabetesPrevalenceAdjustmentFactor:  " +
//          getDiabetesPrevalenceAdjustmentFactor()); //L3b
//  System.out.println("diabetesRemissionWorksheetName:  " +
//          getDiabetesRemissionWorksheetName()); //L34
//  System.out.println("diabetesRemissionAdjustmentFactor:  " +
//          getDiabetesRemissionAdjustmentFactor()); //L4b
//  System.out.println("weightProgressionWorksheetName:  " +
//          getWeightProgressionWorksheetName()); //L5a
//  System.out.println("weightProgressionAdjustmentFactor:  " +
//          getWeightProgressionAdjustmentFactor()); //L5b
//  System.out.println("weightRegressionWorksheetName:  " +
//          getWeightRegressionWorksheetName()); //L6a
//  System.out.println("weightRegressionAdjustmentFactor:  " +
//          getWeightRegressionAdjustmentFactor()); //L6b
//  System.out.println("overrideParameters:  " +
//          getOverrideParameters()); //M1
//  System.out.println("writeOutputFile:  " +
//          getWriteOutputFile()); //M2
//  System.out.println("outputFileName:  " +
//          getOutputFileName()); //M3
//  System.out.println("--------------------------------------");
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

     }//end i1_getInputParameters()
//==============================================================================


//==============================================================================
// Method:  i2_getExcelData()
//------------------------------------------------------------------------------

     private void i2_getExcelData()
     {

    	 // For the Mac OS, use a forward slash for the "data" folder delimeter,
    	 // rather than a backslash, or a slash delimiter "\\".
    	 File dataFile = new File("data/" + getDataWorkbookFileName());
         Workbook dataWorkbook = null;

//    double[][] disabilityIncidenceRate = 
//            new double[Constants.DATA_TABLE_MAXIMUM_AGE][Constants.NUMBER_OF_WEIGHT_CATEGORIES];
//    double[][] disabilityRemissionRate = 
//            new double[Constants.DATA_TABLE_MAXIMUM_AGE][Constants.NUMBER_OF_WEIGHT_CATEGORIES];
//    double[][] weightProgressionRate = 
//            new double[Constants.DATA_TABLE_MAXIMUM_AGE][Constants.NUMBER_OF_WEIGHT_CATEGORIES];
//    double[][] weightResgressionRate = 
//            new double[Constants.DATA_TABLE_MAXIMUM_AGE][Constants.NUMBER_OF_WEIGHT_CATEGORIES];


         // Initialize the data arrays
        for (int i = 0; i < Constants.DATA_TABLE_MAXIMUM_AGE; i++)
             for (int j = 0; j < Constants.NUMBER_OF_WEIGHT_CATEGORIES; j++)
             {
                 diabetesIncidenceRate[i][j] = 0.0;
                 diabetesPrevalenceRate[i][j] = 0.0;
                 diabetesRemissionRate[i][j] = 0.0;
                 weightProgressionRate[i][j] = 0.0;
                 weightRegressionRate[i][j] = 0.0;
             }//end for
         
            try
            {
                dataWorkbook = Workbook.getWorkbook(dataFile);
            } catch (BiffException exception)
            {
                exception.printStackTrace();
            } catch (IOException exception)
            {
                exception.printStackTrace();
            }
             Sheet diabetesIncidenceSheet = dataWorkbook.getSheet(getDiabetesIncidenceWorksheetName());
             Sheet diabetesPrevalenceSheet = dataWorkbook.getSheet(getDiabetesPrevalenceWorksheetName());
             Sheet diabetesRemissionSheet = dataWorkbook.getSheet(getDiabetesRemissionWorksheetName());
             Sheet weightRegressionSheet = dataWorkbook.getSheet(getWeightRegressionWorksheetName());
             Sheet weightProgressionSheet = dataWorkbook.getSheet(getWeightProgressionWorksheetName());
             
             for (int m = 0; m < Constants.DATA_TABLE_MAXIMUM_AGE - Constants.DATA_TABLE_MINIMUM_AGE +1; m++) 
             {
                for (int n = 0; n < Constants.NUMBER_OF_WEIGHT_CATEGORIES; n++) 
                {
                    diabetesIncidenceRate[Constants.DATA_TABLE_MINIMUM_AGE + m-1][n] =
                    Double.valueOf(diabetesIncidenceSheet.getCell(n+1, m+1).getContents());
                    diabetesPrevalenceRate[Constants.DATA_TABLE_MINIMUM_AGE + m-1][n] =
                    Double.valueOf(diabetesPrevalenceSheet.getCell(n+1, m+1).getContents());
                    diabetesRemissionRate[Constants.DATA_TABLE_MINIMUM_AGE + m-1][n] =
                    Double.valueOf(diabetesRemissionSheet.getCell(n+1, m+1).getContents());
                    weightRegressionRate[Constants.DATA_TABLE_MINIMUM_AGE + m-1][n] =
                    Double.valueOf(weightRegressionSheet.getCell(n+1, m+1).getContents());
                    weightProgressionRate[Constants.DATA_TABLE_MINIMUM_AGE + m-1][n] =
                    Double.valueOf(weightProgressionSheet.getCell(n+1, m+1).getContents());
                    
                }//end for
             }//end for          


//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Test 
// This block tests the Excel data variables.
// To run the test, remove comments from the code below.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//    System.out.println("Environment i2_getExcelData() at " +
//       RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
//
//    System.out.println("Disability incidence rates");
//    System.out.println("--------------------------------------");
//    for (int i = 0; i < Constants.DATA_TABLE_MAXIMUM_AGE; i++)
//    {
//         for (int j = 0; j < Constants.NUMBER_OF_WEIGHT_CATEGORIES; j++)
//         {
//             System.out.printf("%8.4f", getDisabilityIncidenceRate()[i][j]);
//         }//end for
//         
//         System.out.print("\n");         
//    }//end for            
//    System.out.println("--------------------------------------");
//
//    System.out.println("Disability prevalence rates");
//    System.out.println("--------------------------------------");
//    for (int i = 0; i < Constants.DATA_TABLE_MAXIMUM_AGE; i++)
//    {
//         for (int j = 0; j < Constants.NUMBER_OF_WEIGHT_CATEGORIES; j++)
//         {
//             System.out.printf("%8.4f", getDisabilityPrevalenceRate()[i][j]);
//         }//end for
//         
//         System.out.print("\n");         
//    }//end for            
//    System.out.println("--------------------------------------");
//
//    System.out.println("Disability remission rates");
//    System.out.println("--------------------------------------");
//    for (int i = 0; i < Constants.DATA_TABLE_MAXIMUM_AGE; i++)
//    {
//         for (int j = 0; j < Constants.NUMBER_OF_WEIGHT_CATEGORIES; j++)
//         {
//             System.out.printf("%8.4f", getDisabilityRemissionRate()[i][j]);
//         }//end for
//         
//         System.out.print("\n");         
//    }//end for            
//    System.out.println("--------------------------------------");
//  
//    System.out.println("Weight regression rates");
//    System.out.println("--------------------------------------");
//    for (int i = 0; i < Constants.DATA_TABLE_MAXIMUM_AGE; i++)
//    {
//         for (int j = 0; j < Constants.NUMBER_OF_WEIGHT_CATEGORIES; j++)
//         {
//             System.out.printf("%8.4f", getWeightRegressionRate()[i][j]);
//         }//end for
//         
//         System.out.print("\n");         
//    }//end for            
//    System.out.println("--------------------------------------");
//  
//    System.out.println("Weight progression rates");
//    System.out.println("--------------------------------------");
//    for (int i = 0; i < Constants.DATA_TABLE_MAXIMUM_AGE; i++)
//    {
//         for (int j = 0; j < Constants.NUMBER_OF_WEIGHT_CATEGORIES; j++)
//         {
//             System.out.printf("%8.4f", getWeightProgressionRate()[i][j]);
//         }//end for
//         
//         System.out.print("\n");         
//    }//end for            
//    System.out.println("--------------------------------------");
//   
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

     }//end i2_getExcelData()
//==============================================================================

//==============================================================================
// Method:  p1_OverrideParameters()
//------------------------------------------------------------------------------
     public void p1_OverrideParameters()
     {
          // Intentionally empty, until needed.
     
     }//end p1_OverrideParameters()
//==============================================================================

//==============================================================================
// Method:  p2_CreateEmployee()
//------------------------------------------------------------------------------
     public static void p2_CreateEmployee(int pIndex, Context<Object> pContext)
     {

            AgentID employeeID = new AgentID(AgentType.Employee, pIndex);
            Employee employee = new Employee(pContext, employeeID);          
            pContext.add(employee);
            
            //------------------------------------------------------------------
            // If the Person distribution type is "Normal", distribute the Persons
            // according to a normal distribution over the Environment.
            // The distribution is limited to the size of the Environment.
            // Note:  a small distance is subtracted from SPACE_SIZE so that agents
            // with coordinates exceeding SPACE_SIZE are displayed.
            //------------------------------------------------------------------
            if (getEmployeeWorkplaceLocationDistributionType().equals(Constants.NORMAL_DISTRIBUTION))
            {
                RandomHelper.createNormal(Constants.WORKPLACE_SPACE_SIZE/2,getEmployeeWorkplaceLocationStandardDeviation());
                xCoordinate = Math.max(Math.min(RandomHelper.getNormal().nextDouble(),
                        Constants.WORKPLACE_SPACE_SIZE - Constants.SMALL_DISTANCE), 0);
                yCoordinate = Math.max(Math.min(RandomHelper.getNormal().nextDouble(),
                        Constants.WORKPLACE_SPACE_SIZE - Constants.SMALL_DISTANCE), 0);
                
                employee.setAgentXLocation(xCoordinate);
                employee.setAgentYLocation(yCoordinate);
                workplaceSpace.moveTo(employee, xCoordinate, yCoordinate);
            }//end if
            
            // If the Person distribution type is "random", distribute the Persons
            // randomly over the environment.
            else
            {
                xCoordinate =
                        RandomHelper.nextDoubleFromTo(0, Constants.WORKPLACE_SPACE_SIZE);
                yCoordinate =
                        RandomHelper.nextDoubleFromTo(0, Constants.WORKPLACE_SPACE_SIZE);
                
                employee.setAgentXLocation(xCoordinate);
                employee.setAgentYLocation(yCoordinate);
                workplaceSpace.moveTo(employee, xCoordinate, yCoordinate);

            }//end else

            // Enter each new Employee in the "Employee list".
            employeeList.add(employee);
            setCurrentMaximumEmployeeIDNumber(pIndex+1);

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Test 
// This outputs data to the console to test the method.
// To run the test, remove comments from the code below.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  System.out.println("Environment p2_CreateEmployee() at " +
//          RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
//  System.out.println("Maximum Employee ID number:  " + getCurrentMaximumEmployeeIDNumber());
//  System.out.println("Size of Employee list:  " + employeeList.size());
//  
//  Iterable<Object> allEmployeesInContext = pContext.getObjects(Employee.class);
//
//     for (Object employee1 : allEmployeesInContext)
//     {
//           System.out.println("Employee ID:  " + ((Employee) employee1).getEmployeeIDString());
//     }
//  System.out.println("--------------------------------------");
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            
     }//end p2_CreateEmployee()
//==============================================================================

//==============================================================================
// Method:  p3_RemoveEmployee()
//          This method is not currently used, because it generates an error
//          that could not be readily resolved.
//          But, because it is a better way to remove Employees, it is left
//          for later investigation.
//------------------------------------------------------------------------------
//     public static void p3_RemoveEmployee(int pEmployeeIDNumber, Context<Object> pContext)
//     {
//
//        ListIterator<Employee> iterator1 = getEmployeeList().listIterator();
//         
//        while (iterator1.hasNext())
//        {
//            Employee employee = iterator1.next();
//            if (employee.getAgentID().getAgentNumber() == pEmployeeIDNumber)
//            {
//                getEmployeeList().remove(employee);
//                pContext.remove(employee);
//            }//end if
//        }//end while
//        
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Test 
// This outputs data to the console to test the method.
// To run the test, remove comments from the code below.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  System.out.println("Environment p3_RemoveEmployee() at " +
//          RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
//  System.out.println("Maximum Employee ID number:  " + getCurrentMaximumEmployeeIDNumber());
//  System.out.println("Size of Employee list:  " + employeeList.size());
//
//  Iterable<Object> allEmployeesInContext = pContext.getObjects(Employee.class);
//
//     for (Object employee1 : allEmployeesInContext)
//     {
//           System.out.println("Employee ID:  " + ((Employee) employee1).getEmployeeIDString());
//     }
//  System.out.println("--------------------------------------");
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        
//     }//end p3_RemoveEmployee()
//==============================================================================

     
     
//==============================================================================
// Methods: getter and setter methods
//------------------------------------------------------------------------------
    public static int getRunNumber()
    {
        return runNumber;
    }

    public static ArrayList<Employee> getEmployeeList()
    {
        return employeeList;
    }

    public static String getEmployeeLabels()
    {
        return employeeLabels;
    }

    public static int getMaximumSimulationPeriods()
    {
        return maximumSimulationPeriods;
    }

     public static int getRandomSeed()
    {
        return randomSeed;
    }

    public static String getProgramDesign_TargetGroup()
    {
        return programDesign_TargetGroup;
    }

     public static IntensityLevel getProgramDesign_Type()
    {
        return programDesign_Type;
    }

    public static double getProgramDesign_TypeLevel1AnnualCost()
    {
        return programDesign_TypeLevel1AnnualCost;
    }

    public static double getProgramDesign_TypeLevel2AnnualCost()
    {
        return programDesign_TypeLevel2AnnualCost;
    }

     public static IntensityLevel getProgramDesign_ChoiceArchitectureIntensity()
    {
        return programDesign_ChoiceArchitectureIntensity;
    }

    public static double getProgramDesign_ChoiceArchitectureLevel1AnnualCost()
    {
        return programDesign_ChoiceArchitectureLevel1AnnualCost;
    }

    public static double getProgramDesign_ChoiceArchitectureLevel2AnnualCost()
    {
        return programDesign_ChoiceArchitectureLevel2AnnualCost;
    }

    public static IntensityLevel getProgramDesign_IncentiveIntensity()
    {
        return programDesign_IncentiveIntensity;
    }

    public static double getProgramDesign_IncentiveLevel1AnnualCost()
    {
        return programDesign_IncentiveLevel1AnnualCost;
    }

    public static double getProgramDesign_IncentiveLevel2AnnualCost()
    {
        return programDesign_IncentiveLevel2AnnualCost;
    }

    public static double getAverageAnnualDiabetesMedicalExpenditure()
    {
        return averageAnnualDiabetesMedicalExpenditure;
    }

    public static int getNumberOfEmployees()
    {
        return numberOfEmployees;
    }

    public static String getEmployeeAgeDistributionType()
    {
        return employeeAgeDistributionType;
    }

    public static int getEmployeeAverageAge()
    {
        return employeeAverageAge;
    }

    public static double getEmployeeAgeStandardDeviation()
    {
        return employeeAgeStandardDeviation;
    }

    public static String getEmployeeWorkplaceLocationDistributionType()
    {
        return employeeWorkplaceLocationDistributionType;
    }

    public static double getEmployeeWorkplaceLocationStandardDeviation()
    {
        return employeeWorkplaceLocationStandardDeviation;
    }

    public static double getEmployeeWorkplaceNeighborhoodRadius()
    {
        return employeeWorkplaceNeighborhoodRadius;
    }

    public static double getEmployeeFirstGoalPercent_Health()
    {
        return employeeFirstGoalPercent_Health;
    }

    public static double getEmployeeFirstGoalPercent_Conformance()
    {
        return employeeFirstGoalPercent_Conformance;
    }

    public static double getEmployeeFirstGoalPercent_Income()
    {
        return employeeFirstGoalPercent_Income;
    }

    public static double getEmployeesNormalWeightPercent()
    {
        return employeesNormalWeightPercent;
    }

    public static double getEmployeesOverweightPercent()
    {
        return employeesOverweightPercent;
    }

    public static double getEmployeesObesePercent()
    {
        return employeesObesePercent;
    }

    public static double getEmployeeStageOfChange_NormalWeight_Stage1Percent()
    {
        return employeeStageOfChange_NormalWeight_Stage1Percent;
    }

     public static double getEmployeeStageOfChange_NormalWeight_Stage2Percent()
    {
        return employeeStageOfChange_NormalWeight_Stage2Percent;
    }

    public static double getEmployeeStageOfChange_NormalWeight_Stage3Percent()
    {
        return employeeStageOfChange_NormalWeight_Stage3Percent;
    }

    public static double getEmployeeStageOfChange_Overweight_Stage1Percent()
    {
        return employeeStageOfChange_Overweight_Stage1Percent;
    }

    public static double getEmployeeStageOfChange_Overweight_Stage2Percent()
    {
        return employeeStageOfChange_Overweight_Stage2Percent;
    }

    public static double getEmployeeStageOfChange_Overweight_Stage3Percent()
    {
        return employeeStageOfChange_Overweight_Stage3Percent;
    }

    public static double getEmployeeStageOfChange_Obese_Stage1Percent()
    {
        return employeeStageOfChange_Obese_Stage1Percent;
    }

    public static double getEmployeeStageOfChange_Obese_Stage2Percent()
    {
        return employeeStageOfChange_Obese_Stage2Percent;
    }

    public static double getEmployeeStageOfChange_Obese_Stage3Percent()
    {
        return employeeStageOfChange_Obese_Stage3Percent;
    }

    public static double getTurnover_BaseProbability()
    {
        return turnover_BaseProbability;
    }

    public static double getTurnover_AdditionalProbability()
    {
        return turnover_AdditionalProbability;
    }

    public static int getRetirementAge()
    {
        return retirementAge;
    }

    public static double getAbsenteeismDays()
    {
        return absenteeismDays;
    }

    public static double getPresenteeismDays()
    {
        return presenteeismDays;
    }

    public static String getDataWorkbookFileName()
    {
        return dataWorkbookFileName;
    }

    public static String getDiabetesIncidenceWorksheetName()
    {
        return diabetesIncidenceWorksheetName;
    }

    public static double getDiabetesIncidenceAdjustmentFactor()
    {
        return diabetesIncidenceAdjustmentFactor;
    }

    public static String getDiabetesPrevalenceWorksheetName()
    {
        return diabetesPrevalenceWorksheetName;
    }

    public static double getDiabetesPrevalenceAdjustmentFactor()
    {
        return diabetesPrevalenceAdjustmentFactor;
    }
    
    public static String getDiabetesRemissionWorksheetName()
    {
        return diabetesRemissionWorksheetName;
    }

    public static double getDiabetesRemissionAdjustmentFactor()
    {
        return diabetesRemissionAdjustmentFactor;
    }

    public static String getWeightProgressionWorksheetName()
    {
        return weightProgressionWorksheetName;
    }

    public static double getWeightProgressionAdjustmentFactor()
    {
        return weightProgressionAdjustmentFactor;
    }

    public static String getWeightRegressionWorksheetName()
    {
        return weightRegressionWorksheetName;
    }

    public static double getWeightRegressionAdjustmentFactor()
    {
        return weightRegressionAdjustmentFactor;
    }

    public static String getOverrideParameters()
    {
        return overrideParameters;
    }

    public static String getWriteOutputFile()
    {
        return writeOutputFile;
    }

    public static String getOutputFileName()
    {
        return outputFileName;
    }

    public static double[][] getDiabetesIncidenceRate()
    {
        return diabetesIncidenceRate;
    }

    public static double[][] getDiabetesPrevalenceRate()
    {
        return diabetesPrevalenceRate;
    }

    public static double[][] getDiabetesRemissionRate()
    {
        return diabetesRemissionRate;
    }

    public static double[][] getWeightRegressionRate()
    {
        return weightRegressionRate;
    }
    
    public static double[][] getWeightProgressionRate()
    {
        return weightProgressionRate;
    }

    public static int getCurrentMaximumEmployeeIDNumber()
    {
        return currentMaximumEmployeeIDNumber;
    }

    public static void setCurrentMaximumEmployeeIDNumber(int pCurrentMaximumEmployeeIDNumber)
    {
        currentMaximumEmployeeIDNumber = pCurrentMaximumEmployeeIDNumber;
    }

//==============================================================================
    
}//end Environment class
