/**
* The Society of Actuaries (SOA) makes this computer model and its computer
* code available to its members for educational purposes. The model and its
* code may be modified and redistributed without prior authorization from the
* SOA. The SOA distributes this computer model and its computer code in the
* hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
* implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
* PURPOSE.
*/ 

package implementation;

//==============================================================================
//Import statements
//------------------------------------------------------------------------------
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import agents.Employee;
import agents.Environment;
import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ISchedule;
import repast.simphony.engine.schedule.ScheduleParameters;

/**
* @author       Alan Mills
* @version      1.0
*/
public class Schedule
{
//==============================================================================
// Variables used in this class
//------------------------------------------------------------------------------
    private static int period;
    public static FileWriter outputFile = null;
    public static BufferedWriter  outputBuffer = null;
    private Context<Object> simulationContext;
//==============================================================================
        
//==============================================================================
// Method:  Schedule() constructor
//------------------------------------------------------------------------------
    /**
     * Constructs the Schedule class.
     *
     * The constructor establishes the simulation's schedule.
     */
    public Schedule(Context<Object> pContext)
    {
        simulationContext = pContext;
        
        ISchedule schedule = RunEnvironment.getInstance().getCurrentSchedule();
        
        ScheduleParameters scheduleTime000 = ScheduleParameters.createRepeating(0.00, 1);
        ScheduleParameters scheduleTime100 = ScheduleParameters.createRepeating(.100, 1);
        ScheduleParameters scheduleTime999 = ScheduleParameters.createRepeating(.999, 1);
        ScheduleParameters scheduleTimeStop = ScheduleParameters.createAtEnd(ScheduleParameters.LAST_PRIORITY);

        schedule.schedule(scheduleTime000, this, "schedule_EmployeeBehaviors");
        schedule.schedule(scheduleTime000, this, "schedule_EnvironmentBehaviors");
        
        // Initialize the static variable "period"
        period = 0;

        schedule.schedule(scheduleTime100, this, "init_IncrementPeriod");
        schedule.schedule(scheduleTimeStop, this, "cleanupAtEnd");  
        
        if (Environment.getWriteOutputFile().matches("Yes"))
        {
            //----------------------------------------------------------------------
            // If the input parameter "M2. Write output file" is "Yes",
            // set up the file to capture output, and write out a file.
            // The character stream class "BufferedWriter" is used to 
            // handle the character data.
            //---------------------------------------------------------------------- 
            String fileName = "output\\" + Environment.getOutputFileName();
            final boolean append = true;
            if (outputFile == null || outputBuffer == null)
            {
                try
                {
                    outputFile = new FileWriter(fileName, append);
                    outputBuffer = new BufferedWriter(outputFile);
                } catch (IOException exception)
                {
                    System.out.println("Encountered an IO exception problem.");
                    System.out.println("writeOutput()");
                    System.out.println("Run: " + Environment.getRunNumber());
                    System.out.println("Period: " + period);
                    exception.printStackTrace();
                }//end catch
            }//end if
            
            schedule.schedule(scheduleTime999, this, "writeOutput");

        }//end if   

    }//end Schedule()
//------------------------------------------------------------------------------


//<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
//<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
// File output
// The next method is for capturing output generated from within the model.
// It is activated only if the input parameter "S2. Write output file" parameter
// is "Yes".
//<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
//<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

//==============================================================================
// Method:  writeOutput()
// This method is used for situations that the Repast "parameter sweep"
// feature cannot handle (which is rare. It writes out a custom-generated file,
// an example of which is given below.
//------------------------------------------------------------------------------
    /**
     * Write out internally-generated output.
     * 
     */
        public void writeOutput()
        {

            // Intentionally empty, until needed
            
        }//end writeOutput()
//==============================================================================

        
//==============================================================================
// Method:  schedule_EnvironmentBehaviors()
//------------------------------------------------------------------------------
    /**
     * Schedule behaviors for the Environment.
     * 
     */
        public void schedule_EnvironmentBehaviors()
        {
                
            Iterable<Object> environmentInContext = simulationContext.getObjects(Environment.class);

            for (Object environment : environmentInContext)
            {
                ISchedule schedule = RunEnvironment.getInstance().getCurrentSchedule();
                
                double tickCount = RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
        
                ScheduleParameters scheduleTime050 = ScheduleParameters.createOneTime(.050 + tickCount);
                
                schedule.schedule(scheduleTime050, environment, "i1_getInputParameters");

            }//end for

            
        }//end schedule_EnvironmentBehaviors()
        
//==============================================================================
// Method:  schedule_EmployeeBehaviors()
//------------------------------------------------------------------------------
    /**
     * Schedule behaviors for Employees in the current simulation context.
     * 
     */
        public void schedule_EmployeeBehaviors()
        {
                
            Iterable<Object> allEmployeesInContext = simulationContext.getObjects(Employee.class);

            for (Object employee : allEmployeesInContext)
            {
                ISchedule schedule = RunEnvironment.getInstance().getCurrentSchedule();
                
                double tickCount = RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
        
                ScheduleParameters scheduleTime100 = ScheduleParameters.createOneTime(.100 + tickCount);
                ScheduleParameters scheduleTime101 = ScheduleParameters.createOneTime(.101 + tickCount);
                ScheduleParameters scheduleTime200 = ScheduleParameters.createOneTime(.200 + tickCount);       
                ScheduleParameters scheduleTime300 = ScheduleParameters.createOneTime(.300 + tickCount);
                ScheduleParameters scheduleTime301 = ScheduleParameters.createOneTime(.301 + tickCount);
                ScheduleParameters scheduleTime302 = ScheduleParameters.createOneTime(.302 + tickCount);
                ScheduleParameters scheduleTime303 = ScheduleParameters.createOneTime(.303 + tickCount);
                ScheduleParameters scheduleTime900 = ScheduleParameters.createOneTime(.900 + tickCount);
                
                schedule.schedule(scheduleTime100, employee, "init_InitializeVariables_BOY");
                schedule.schedule(scheduleTime101, employee, "p1_MakeParticipationDecision");
                schedule.schedule(scheduleTime200, employee, "p2_MakeComplianceDecision");
                schedule.schedule(scheduleTime300, employee, "p3_DetermineWeightCategory");
                schedule.schedule(scheduleTime301, employee, "p4_DetermineDiseaseStatus");
                schedule.schedule(scheduleTime302, employee, "p5_TerminateEmployment");
                schedule.schedule(scheduleTime303, employee, "p6_Retire");
                schedule.schedule(scheduleTime900, employee, "init_IncrementVariables_EOY");
            }//end for

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Test 
// This outputs data to the console, to test this process.
// To run the test, remove comments from the code below.
// It is best to run the test with a small number of Employee agents.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  System.out.println("Schedule schedule_EmployeeBehaviors() at " +
//          RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
//  System.out.println("Size of Employee list:  " + Environment.getEmployeeList().size());
//  
//  Iterable<Object> allEmployeesInContext1 = simulationContext.getObjects(Employee.class);
//
//     for (Object employee1 : allEmployeesInContext1)
//     {
//           System.out.println("Employee ID:  " + ((Employee) employee1).getEmployeeIDString());
//     }
//  System.out.println("--------------------------------------");
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            
        }//end schedule_EmployeeBehaviors()
//==============================================================================
        
        
        
//==============================================================================
// Method:  init_IncrementPeriod()
//------------------------------------------------------------------------------    
    /**
     * Increment the period counter.
     * 
     */
        public void init_IncrementPeriod()
        {
            period++;

            if (period <= Environment.getMaximumSimulationPeriods())
            {
                System.out.println("Period: " + period); //Useful for batch runs
//              System.out.println("--------------------------------------");
            }//end if
            else
            {
                RunEnvironment.getInstance().endRun();
                cleanupAtEnd();
            }//end else
                
        }//end init_IncrementPeriod()
//==============================================================================

//==============================================================================
// Method:  cleanupAtEnd()
//------------------------------------------------------------------------------
    /**
     * At the end of the simulation, flush and close the buffer, and
     * close the file.
     * 
     */
        public static void cleanupAtEnd()
        {
                    
            try
            {
                if (outputBuffer != null)
                {
                    outputBuffer.flush();
                    outputBuffer.close();
                }//end if
                if (outputFile != null)
                {
                    outputFile.close();
                }//end if
                outputBuffer = null;
                outputFile = null;
            } catch (IOException exception)
            {
                System.out.println("Encountered an IO exception problem.");
                System.out.println("cleanupAtEnd()");
                System.out.println("Run: " + Environment.getRunNumber());
                System.out.println("Period: " + period);
                exception.printStackTrace();
            }//end catch
                    
        }//end cleanupAtEnd()
//==============================================================================


//==============================================================================
// Methods:  getter and setter methods
//------------------------------------------------------------------------------
    /**
     * Gets the period.
     * 
     */
    public static int getPeriod()
    {
        
        return period;
        
    }//end getPeriod()

//==============================================================================

}//end Schedule class
