/**
* The Society of Actuaries (SOA) makes this computer model and its computer
* code available to its members for educational purposes. The model and its
* code may be modified and redistributed without prior authorization from the
* SOA. The SOA distributes this computer model and its computer code in the
* hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
* implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
* PURPOSE.
*/

package implementation;
import agents.Environment;
import repast.simphony.context.Context;
import repast.simphony.context.DefaultContext;
import repast.simphony.context.space.continuous.ContinuousSpaceFactory;
import repast.simphony.context.space.continuous.ContinuousSpaceFactoryFinder;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.continuous.SimpleCartesianAdder;
import resources.Constants;

//==============================================================================
//Import statements
//------------------------------------------------------------------------------

/**
 * @author      Alan Mills
 * <p>
 * This class is where the simulation starts.
 * It sets up the Repast "context" and "projections" for the simulation.
 * The context holds the Employee agents, and the main projection is continuous
 * space at the Employer workplace.
 * @version     1.0
 */
public class SimulationContext extends DefaultContext<Object> implements ContextBuilder<Object>
{

    //==============================================================================
    // Method:  build
    // Note:    This class is where the simulation starts.
    //          Its constructor overrides a Repast method. 
    //------------------------------------------------------------------------------
     /**
      * Establishes the simulation's "context" and instantiates the Environment.
      * This is where the simulation starts.
      */
     @Override
     public Context<Object> build(Context<Object> pContext)
     {
         pContext.setId(Constants.CONTEXT_ID);
         
         // Create the "workplace" continuous space projection and add it to the context
         ContinuousSpaceFactory spaceFactory = ContinuousSpaceFactoryFinder
                 .createContinuousSpaceFactory(null);        
         
         ContinuousSpace<Object> workplaceSpace = spaceFactory.createContinuousSpace
         (
             Constants.WORKPLACE_SPACE_ID,
             pContext,
             new SimpleCartesianAdder<Object>(),
             new repast.simphony.space.continuous.WrapAroundBorders(),
             Constants.WORKPLACE_SPACE_SIZE,
             Constants.WORKPLACE_SPACE_SIZE
         );//end .createContinuousSpace()         


         //Establish the Environment, and add it to the context.
         Environment environment;
         environment = null;
         
         environment = new Environment(workplaceSpace, pContext);

         pContext.add(environment);

         return pContext;
         
     }//end build()
     
}//end SimulationContexts class
