/**
* The Society of Actuaries (SOA) makes this computer model and its computer
* code available to its members for educational purposes. The model and its
* code may be modified and redistributed without prior authorization from the
* SOA. The SOA distributes this computer model and its computer code in the
* hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
* implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
* PURPOSE.
*/ 

package resources;

/**
 * @author      Alan Mills
 * <p>
 * This class defines string and numerical constants used throughout the simulation.
 * @version     1.0
 */
public class Constants
{
    /**
     * Constant identifying the environment context.
     */
    public static final String CONTEXT_ID = "Environment";
    
    /**
     * Constant identifying the workplace space projection.
     */
    public static final String WORKPLACE_SPACE_ID = "Workplace";
    
    /**
     * Constant to initialize the workplace space dimension.
     */
    public static final int WORKPLACE_SPACE_SIZE = 100;
    
     /**
     * Constant specifying a small distance.
     */
    public static final double SMALL_DISTANCE = .00001;
        
    /**
     * Constant specifying the normal distribution.
     */
    public static final String NORMAL_DISTRIBUTION = "Random normal distribution";
    
    /**
     * Constant specifying the uniform distribution.
     */
    public static final String UNIFORM_DISTRIBUTION = "Random uniform distribution";

    /**
     * Constant specifying the number of weight categories in data tables.
     */
    public static final int NUMBER_OF_WEIGHT_CATEGORIES = 6;

    /**
     * Constant specifying the minimum age in data tables.
     */
    public static final int DATA_TABLE_MINIMUM_AGE = 25;

    /**
     * Constant specifying the maximum age in data tables.
     */
    public static final int DATA_TABLE_MAXIMUM_AGE = 75;
    
    /**
     * Constant specifying the first column in the data tables
     */
    public static final int NORMAL_WITH_EXERCISE = 0;
        
    /**
     * Constant specifying the second column in the data tables
     */
    public static final int NORMAL_WITHOUT_EXERCISE = 1;
            
    /**
     * Constant specifying the third column in the data tables
     */
    public static final int OVERWEIGHT_WITH_EXERCISE = 2;
        
    /**
     * Constant specifying the fourth column in the data tables
     */
    public static final int OVERWEIGHT_WITHOUT_EXERCISE = 3;    
    
    /**
     * Constant specifying the fifth column in the data tables
     */
    public static final int OBESE_WITH_EXERCISE = 4;
        
    /**
     * Constant specifying the sixth column in the data tables
     */
    public static final int OBESE_WITHOUT_EXERCISE = 5;    

    
    /**
     * Constant for the average BMI of an Employee with normal weight.
     */
    public static final double BMI_NORMAL_WEIGHT = 21.75;    

   /**
     * Constant for the average BMI of an overweight Employee.
     */
    public static final double BMI_OVERWEIGHT = 27.50;
    
   /**
     * Constant for the average BMI of an obese Employee.
     */
    public static final double BMI_OBESE = 34;
    
    
    public enum AgentType {Employee, None};
    public enum GoalDescription {Health, Conformance, Income, None};
    public enum Disease {Diabetes, None};
    public enum StageOfChange {Stage1, Stage2, Stage3, None};
    public enum WeightCategory {Normal, Overweight, Obese, None};
    public enum IntensityLevel {None, Level1, Level2};
        
//==============================================================================
// Method:  Constants() constructor
// Note:    The purpose of this constructor is to prevent a compiler error.
//------------------------------------------------------------------------------
    private Constants()
    {
        ; //empty statement
    }//end Constants()
//==============================================================================
    
}//end Constants class
