/**
* The Society of Actuaries (SOA) makes this computer model and its computer
* code available to its members for educational purposes. The model and its
* code may be modified and redistributed without prior authorization from the
* SOA. The SOA distributes this computer model and its computer code in the
* hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
* implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
* PURPOSE.
*/

package styles;

//==============================================================================
//Import statements
//------------------------------------------------------------------------------
import java.awt.Color;
import java.awt.Font;
import agents.Employee;
import agents.Environment;
import repast.simphony.visualizationOGL2D.StyleOGL2D;
import saf.v3d.ShapeFactory2D;
import saf.v3d.scene.Position;
import saf.v3d.scene.VSpatial;

/**
 * @author      Alan Mills
 * @version     1.0
 */
public class EmployeeStyle implements StyleOGL2D<Employee>
{
    private ShapeFactory2D shapeFactory;

//==============================================================================
// Method: These first methods implement the label methods of StyleOGL2D.
//------------------------------------------------------------------------------
    @Override
    public String getLabel(Employee employee)
    {
        String labelSwitch = Environment.getEmployeeLabels();  
        if (labelSwitch.equals("Yes")) return employee.getEmployeeIDString();
        else return ""; 
    }

    @Override
    public Font getLabelFont(Employee employee)
    {
        return new Font("sansserif", Font.BOLD, 12);
    }

    @Override
    public float getLabelXOffset(Employee employee)
    {
        return 0;
    }

    @Override
    public float getLabelYOffset(Employee employee)
    {
        return 0;
    }

    @Override
    public Position getLabelPosition(Employee employee)
    {
        return Position.SOUTH;
    }

    @Override
    public Color getLabelColor(Employee employee)
    {
        return new Color (0, 0, 0); //Black
    }
//==============================================================================


//==============================================================================
// This method sets the shape of the icon representing the Employee to a circle.
//------------------------------------------------------------------------------
    @Override
    public VSpatial getVSpatial(Employee employee, VSpatial spatial)
    {
        
         return shapeFactory.createCircle(5, 100);
    }

//==============================================================================
// This method sets the color of the disk representing the Employee.
// The color is given by the triplet (Red, Green, Blue),
// where Red, Green, and Blue are integers between 0 and 255.
// Colors for Repast are represented in the standard Red/Green/Blue format.
//------------------------------------------------------------------------------
    @Override
    public Color getColor(Employee employee)
    {

        if (employee.getCurrentEmploymentStatus() == false || employee.getCurrentRetirementStatus() == true)
            return new Color (255, 255, 255); //White
        else
        {        
            switch (employee.getCurrentDiseaseStatus())
            {
                case None:  return new Color (220, 220, 220);       //Light grey
                case Diabetes:  return new Color (255, 0, 0);       //Red
                default: return new Color (255, 255, 255);          //White
            }//end switch
        }//end else
    }

//==============================================================================
// This method sets the size of the disk representing the Employee.
// The heavier the Employee, the larger the disk.
//------------------------------------------------------------------------------
    @Override
    public float getScale(Employee employee)
    {

            switch (employee.getCurrentWeightCategory())
            {
                case Normal:  return (float) 1.0;
                case Overweight:  return (float) 2.0;
                case Obese:  return (float) 3.0;
                default: return (float) 1.0;
            }            
    }


//==============================================================================
// These methods set the size of the disk representing the Employee.
// If the employee is sick, the size is larger.
//------------------------------------------------------------------------------
    @Override
    public int getBorderSize(Employee employee)
    {
        if (employee.getCurrentEmploymentStatus() == false || employee.getCurrentRetirementStatus() == true)
            return 2;
        else
        {
             if (employee.getCurrentProgramParticipationStatus() == true)
                return 4;
             else return 2;
        }//end else
    }

    @Override
    public Color getBorderColor(Employee employee)
    {

        if (employee.getCurrentEmploymentStatus() == false || employee.getCurrentRetirementStatus() == true)
            return new Color (120, 120, 120); //Dark grey
        else
        {
            if (employee.getCurrentProgramParticipationStatus() == true)
                return new Color (0, 0, 255); //Blue
            else return new Color (120, 120, 120);   //Dark grey
        }//end else
    }

//==============================================================================
// Method: These are general methods implementing StyleOGL2D.
//------------------------------------------------------------------------------
    @Override
    public void init(ShapeFactory2D shapeFactory)
    {
        this.shapeFactory = shapeFactory;
    }

    @Override
    public float getRotation(Employee employee)
    {
        return 0;
    }
    
}//end EmployeeStyle class
